import numpy as np
import pyqtgraph as pg

from .theme import pg_legend_style


class ImageDescription:

    def __init__(self, name, title=None, xlabel=None, ylabel=None, image=None, colormap=None, levels=None):
        self.name = name
        self.title = title
        self.xlabel = xlabel
        self.ylabel = ylabel
        self.image = image
        self.colormap = colormap
        self.levels = levels

    @property
    def data(self):
        return np.asarray(self.image)

    @data.setter
    def data(self, value):
        self.image = value

    def append(self, xy):
        print("ignored image append")

    def extend(self, data):
        print("ignored image extend")


    def make_plot(self, plotwidget, style):
        if self.levels:
            res = plotwidget.setImage(self.data, levels = self.levels) #Set levels for histogram
        else:
            res = plotwidget.setImage(self.data)    

        if self.title:
            plotwidget.setTitle(self.title)

        if self.xlabel:
            vbox = plotwidget.getView()
            vbox.addItem(pg.LabelItem(self.xlabel, size='50pt'))

        if self.ylabel:
            plotwidget.setLabel("left", self.ylabel)

        if self.colormap:
            cm = pg.colormap.get(self.colormap)
            plotwidget.setColorMap(cm)

        return res


    def to_dict(self):
        return {k: v for k, v in self.__dict__.items() if not k.startswith("_") and k != "name" and v is not None}



