import pyqtgraph as pg
from .mdisubwin import MDISubWindow
from ..theme import pg_plot_style
from PyQt5.QtWidgets import QMainWindow, QWidget, QPushButton, QGridLayout, QVBoxLayout, QApplication, QSlider
from PyQt5.QtCore import Qt, QThread, QTimer

class MDISubImageBase(MDISubWindow):
    def __init__(self, name, *args, **kwargs):
        super().__init__(name, *args, **kwargs)


class MDISubImage(MDISubImageBase):

    def __init__(self, name, desc, *args, **kwargs):
        super().__init__(name, *args, **kwargs)
        
        self.pw = pw = pg.ImageView()
        self.setWidget(pw)

        # connect to plot mouse-over event
        pw.scene.sigMouseMoved.connect(self.on_hover)

        style = pg_plot_style()
        # cm = pg.colormap.get('CET-C1')
        # pw.setColorMap(cm)

        plot = desc.make_plot(self.pw, style)
        self.plots = {name: plot}
        self.image = desc.data

    def on_hover(self, event):
        coord = self.pw.imageItem.mapFromScene(event)
        x = coord.x()
        y = coord.y()
        x = int(x)
        y = int(y)
        try:
            z = self.image[x, y]
        except IndexError:
            return
        z = round(z, 3)
        self.setToolTip(f"x = {x}\ny = {y}\nz = {z}")


class MDISubMultiImage(MDISubImageBase):

    def __init__(self, name, descs, *args, **kwargs):
        super().__init__(name, *args, **kwargs)
        
        self.pw = pw = pg.ImageView()

        self.central_widget = QWidget()
        self.button_next = QPushButton('Next', self.central_widget)
        self.button_previous = QPushButton('Previous', self.central_widget)
        self.slider = QSlider(Qt.Horizontal)
        self.slider.setRange(0,len(descs)-1)

        self._create_layout()
        self._connect_signals()

        names = [names for names, _ in descs.items()]
        self.descriptions = [desc for _, desc in descs.items()]
        desc = self.descriptions[0]

        style = pg_plot_style()
        plot = desc.make_plot(self.pw, style)
        self.plots = {name: plot}
        self.image = desc.data

    def _create_layout(self):
        self.layout = QGridLayout(self.central_widget)
        self.layout.addWidget(self.button_next,1,2)
        self.layout.addWidget(self.button_previous, 1,0)
        self.layout.addWidget(self.pw, 0,0, 1,3)
        self.layout.addWidget(self.slider, 1,1)
        self.setWidget(self.central_widget)

    def _connect_signals(self):
        self.button_next.clicked.connect(lambda:self.update_slider(1))
        self.button_previous.clicked.connect(lambda:self.update_slider(-1))
        self.slider.valueChanged.connect(self.update_img)
        
    def update_slider(self, value):
        self.slider.setValue(self.slider.value()+value)

    def update_img(self, value):
        self.pw.setImage(self.descriptions[value].data)
        self.set_title_in_image()

    def set_title_in_image(self):
        if self.descriptions[self.slider.value()].xlabel:
            vbox = self.pw.getView()
            allchildren = vbox.allChildren()
            for child in allchildren: 
                if isinstance(child, pg.LabelItem):
                    vbox.removeItem(child)
            vbox.addItem(pg.LabelItem(self.descriptions[self.slider.value()].xlabel, size='50pt'))
