from grum.rpc import RPCClient
from os import listdir
from os.path import isfile, join
import h5py
import numpy as np
import argparse

class plotPhases:
    """
    A possibility to plot reconstructions in a scan number range
    """
    def __init__(self, start, end, step=1):
        self.start = start
        self.end = end
        self.step =step
        self.eaccount = "e20631"
        self.grum_client = RPCClient("localhost", 8000)   # this is outgoing
        # self.image_description_list = [] 

    def plot_all_phases(self):

        for scan_nr in range(self.start, self.end + 1, self.step):
            print("retrieving data for scan: ", scan_nr)
            ending_path = self._get_scan_dir(1000, scan_nr, leading_zeros = 5)
            mypath = "/sls/X12SA/data/" + self.eaccount + "/Data10/analysis/" + ending_path           
            recon_file = self.get_recon_file(mypath)
            with h5py.File(mypath + '/' + recon_file, 'r') as hf:
                recon_img_phase = self.get_recon_phase(hf)
            
            xlab = str(scan_nr)
            print("appending scan nr: ", scan_nr)
            self.grum_client.append_image_to_list([f"{scan_nr} Phase", {'image': recon_img_phase, "xlabel": xlab}])
            # self.image_description_list.append([f"{scan_nr} Phase", {'image': recon_img_phase, "xlabel": xlab}])

        # print("calling plot images from list")
        # self.grum_client.append_images_to_list(self.image_description_list)        

    def get_recon_phase(self, hf):
    
        recon_object = hf.get('reconstruction/object')

        recon_object = np.array(recon_object)
        recon_object_angle = np.angle(recon_object)
        
        recon_object_angle = np.rot90(recon_object_angle, 3)
        recon_img_angle = recon_object_angle.tolist()
        return recon_img_angle
    
    def get_recon_file(self, mypath):   
        files = [f for f in listdir(mypath) if isfile(join(mypath, f))]
        for file in files:
            if file.endswith("recons.h5"):
                return file
        print("no file for this scan nr")        
        return None
    
    def _get_scan_dir(self, scan_bundle, scan_number, leading_zeros=None):
        if leading_zeros is None:
            leading_zeros = len(str(scan_bundle))
        floor_dir = scan_number // scan_bundle * scan_bundle
        return f"S{floor_dir:0{leading_zeros}d}-{floor_dir+scan_bundle-1:0{leading_zeros}d}/S{scan_number:0{leading_zeros}d}"
    

if __name__ == "__main__":
    start = 5037 # default
    end = 5038 # default
    step = 1
    
    parser = argparse.ArgumentParser(description="description", formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument("-s", "--startend", nargs =2, help="set start, end")
    clargs = parser.parse_args()
    if clargs:
        start = int(clargs.startend[0])
        end = int(clargs.startend[1])
    
    print('starts plotting phase for scans: ',start, ' to ', end)
    pp = plotPhases(start, end)
    pp.plot_all_phases()